(defpackage :male
  (:use :common-lisp :ltk)
  (:export #:male))

(in-package :male)

(defparameter *nombre-programa* "Male")
(defparameter *acronimo* "Mximo anlisis de la lectura espectral")
(defparameter *version* 0.2)
(defparameter *ao* 2017)
(defparameter *autor* "Luis G. Lpez")
(defparameter *creditos* (concatenate 'string 
				      (format nil "~15t~a - v~a (~a) ~%* ~a *~%~% ~40t por ~a" 
					      *nombre-programa* *version* *ao* *acronimo* *autor*)))

(defun leer (widget)
  (read-from-string (text widget)))

(defun male (&optional (n 300) (a 3000) (b 800) (lamda 550) (d 2000) (max-x 800) (factor-i 0.5))
  (with-ltk ()
    (let* ((marco-controles (make-instance 'frame :pack '(:side :top :fill :x)))
	   (marco-n (make-instance 'labelframe :text "N" :master marco-controles :pack '(:side :left :fill :both)))
	   (n-des (make-instance 'spinbox :master marco-n :from 1 :to 1000000 :text (format nil "~a" n) :pack '(:side :top)))
	   (marco-d (make-instance 'labelframe :text "distancia pantalla (pixeles)" :master marco-controles :pack '(:side :left :fill :both)))
	   (d-des (make-instance 'spinbox :master marco-d :from 1 :to 1000000 :text (format nil "~a" d) :pack '(:side :top)))
	   (marco-ancho (make-instance 'labelframe :text "ancho pantalla (pixeles)" :master marco-controles :pack '(:side :left :fill :both)))
	   (ancho-des (make-instance 'spinbox :master marco-ancho :from 1 :to 4000 :text (format nil "~a" (* 2 max-x)) :pack '(:side :top)))
	   (marco-b (make-instance 'labelframe :text "ancho rendijas (nm)" :master marco-controles :pack '(:side :left :fill :both)))
	   (b-des (make-instance 'spinbox :master marco-b :from 380 :to 100000 :text (format nil "~a" b) :pack '(:side :top)))
	   (marco-a (make-instance 'labelframe :text "separacin rendijas (nm)" :master marco-controles :pack '(:side :left :fill :both)))
	   (a-des (make-instance 'spinbox :master marco-a :from 380 :to 100000 :text (format nil "~a" a) :pack '(:side :top)))
	   (marco-lamda (make-instance 'labelframe :text "lambda (nm)" :master marco-controles :pack '(:side :left :fill :both)))
	   (lamda-des (make-instance 'spinbox :master marco-lamda :from 380 :to 780 :text (format nil "~a" lamda) :pack '(:side :top)))
	   (blanca? (make-instance 'check-button :text "Luz blanca?" :master marco-controles :pack '(:side :left :padx 5))) 
	   (marco-i (make-instance 'labelframe :text "Intensidad" :master marco-controles :pack '(:side :left :fill :both)))
	   (i-des (make-instance 'scale :master marco-i :from 0 :to 1 :pack '(:side :top)))
	   (marco-imagen (make-instance 'labelframe :text "Imagen" :pack '(:side :top :fill :x)))
	   (sc-imagen    (make-instance 'scrolled-canvas :master marco-imagen  :pack '(:side :top :fill :x)))
	   (lienzo-imagen (canvas sc-imagen))
	   (marco-perfil  (make-instance 'labelframe :text "Intensidad" :pack '(:side :bottom :fill :both)))
	   (sc-perfil    (make-instance 'scrolled-canvas :master marco-perfil :pack '(:side :top :fill :both :expand 1)))
	   (lienzo-perfil (canvas sc-perfil)))
      (make-instance 'button :text "Ayuda" :master marco-controles :pack '(:side :right)
		     :command (lambda () (do-msg *creditos* :title "Crditos")))
      (wm-title *tk* (format nil "~a - v~a (~a)" *nombre-programa* *version* *ao*))
      (setf (value i-des) factor-i)
      (labels ((dibujar () 
		 (let ((max-x (/ (leer ancho-des) 2)))
		   (dibujar-espectro (leer n-des) (leer a-des) (leer b-des) (leer lamda-des) (leer d-des)
				     (- max-x) max-x lienzo-imagen lienzo-perfil (value blanca?) (value i-des)))))
	(setf (command n-des) (lambda (valor) (declare (ignore valor)) (dibujar))
	      (command lamda-des) (lambda (valor) (declare (ignore valor)) (dibujar))
	      (command d-des) (lambda (valor) (declare (ignore valor)) (dibujar))
	      (command a-des) (lambda (valor) 
				(if (< (read-from-string valor) (leer b-des))
				    (setf (text a-des) (format nil "~a" (leer b-des)))
				    (dibujar)))
	      (command b-des) (lambda (valor) 
				(if (< (leer a-des) (read-from-string valor))
				    (setf (text b-des) (format nil "~a" (leer a-des)))
				    (dibujar)))
	      (command ancho-des) (lambda (valor) (declare (ignore valor)) (dibujar))
	      (command blanca?) (lambda (valor)
				  (configure lamda-des "state" (if (= 1 valor) "disabled" "normal"))
				  (dibujar)))
	(bind i-des "<ButtonRelease-1>" (lambda (evt) (declare (ignore evt)) (dibujar)))
	(dibujar)))))
  
(defun dibujar-espectro (n a b lamda d min-x max-x lienzo-imagen lienzo-perfil blanca? factor-i)
  (let* ((lista-puntos (make-list (* 2 (- max-x min-x -1))))
	 (i-max (* n n))
	 (i-max-blanca (if blanca? (* i-max 400) i-max))
	 (lambda-1 (if blanca? 380 lamda))
	 (lambda-2 (if blanca? 781 (1+ lamda))))
    (clear lienzo-imagen)
    (do ((x min-x (+ x 1)))
	((> x max-x))
      (let ((linea (create-line* lienzo-imagen x 0 x 200))
	    (color (list 0 0 0))
	    (i-total 0))
	(loop for lam from lambda-1 below lambda-2 do
	     (let* ((k (/ pi lam))
		    (intensidad (* factor-i (i n a b (x->seno-theta x d) k))))
	       (setf color (mapcar #'+ color (lambda->rgb lam (/ intensidad i-max))))
	       (incf i-total intensidad)))
	(itemconfigure lienzo-imagen linea "fill" (format nil "#~6,'0x" (min #xffffff (rgb->n color))))
	(let ((i (* 2 (- x min-x))))
	  (setf (nth i lista-puntos) x
		(nth (1+ i) lista-puntos) (- 500 (* 500 (/ i-total i-max-blanca)))))))
    (clear lienzo-perfil)
    (create-line lienzo-perfil lista-puntos))
  (calc-scroll-region lienzo-imagen)
  (calc-scroll-region lienzo-perfil))

(defun i (n a b seno-theta k)
  (let ((alfa (* k a seno-theta))
	(beta (* k b seno-theta)))
    (* (if (= 0 beta) 1 (expt (/ (sin beta) beta) 2))
       (if (= 0 alfa) (* n n) (expt (/ (sin (* n alfa)) (sin alfa)) 2)))))

(defun x->seno-theta (x d)
  (let ((tan-theta (/ x d)))
    (/ tan-theta (sqrt (+ 1 (expt tan-theta 2))))))
  
(defun lambda->rgb (lambda intensidad)
  (if (<= 380.0 lambda 780.0)
      (let* ((rgb-1 (cond 
		      ((< lambda 440) (list (/ (- 440.0 lambda) 60.0) 0.0 1.0))
		      ((< lambda 490) (list 0.0 (/ (- lambda 440.0) 50.0) 1.0))
		      ((< lambda 510) (list 0.0 1.0 (/ (- 510.0 lambda) 20.0)))
		      ((< lambda 580) (list (/ (- lambda 510.0) 70.0) 1.0 0.0))
		      ((< lambda 645) (list 1.0 (/ (- 645.0 lambda) 65.0) 0.0))
		      (t              (list 1.0 0.0 0.0))))
	     (f     (cond
		      ((< lambda 420)  (+ 0.3 (* 0.0175 (- lambda 380))))
		      ((<= lambda 700) 1.0)
		      (t               (+ 0.3 (* 0.00875 (- 780 lambda)))))))
	(mapcar #'(lambda (c) (min 255 (round (* c f intensidad 255)))) rgb-1))
      (list 0 0 0)))

(defun rgb->n (rgb-255)
  (+ (* 65536 (first rgb-255)) (* 256 (second rgb-255)) (third rgb-255)))

(defun lambda->n (lambda intensidad)
  (rgb->n (lambda->rgb lambda intensidad)))


